package ScoreBoard;
use strict;
use Carp;

use IO::File;
use Date::Manip;
$ScoreBoard::VERSION = '0.5';

################# Configuration part ###################
#This configuration will be used by all instances of the module.

my $points_per_kill = 1;
my $points_for_suicide = -1;

#Now the weapon translation hash
#This one is used for Half-life Counterstrike
my %trans = ( "ak47" => "AK-47",
	      "grenade" => "Granat",
              "deagle" => "Desert Eagle",
              "m3" => "Shotgun (Benelli M3 Super90)",
              "awp" => "AI Artic Warfare/Magnum",
              "usp" => "H&K USP .45 Tactical",
              "mp5navy" => "H&K MP5-Navy",
              "xm1014" => "Auto Shotgun (Benelli XM1014)",
              "g3sg1" => "H&K G3/SG-1 Sniper Rifle",
              "knife" => "Kniv",
              "glock18" => "Glock 18",
              "tmp" => "Steyr Tactical Machine Pistol",
	      "p90" => "FN P90",
              "aug" => "Steyr Aug",
              "m4a1" => "Colt M4A1 Carbine",
              "world" => "Selvmord",
              "m249" => "FN M249 Para",
              "p228" => "SIG Sauer (P228)",
              "mac10" => "UZI (Ingram MAC-10)",
              "sg552" => "Sig SG-552 Commando",
              "scout" => "Steyr Scout",
              "tank" => "Tank"
            );

my %alias = ( "g" => "ScaryIgor",
              "Den_Svoemmende_Sniper" => "ScaryIgor",
              "DENSTOREIDIOTMEDKNIV" => "Shitman",
              "[AD&D]MC_Beat" => "[AD&D]Mc_Beat",
              "(" => "ScaryIgor",
	      "(1)Funky_Boris" => "Andre",
              "(1)ProGuy" => "Andre",
              "(2)Funky_Boris" => "Andre",
              "Camper svinet" => "Expert",
              "Camper" => "Expert",
              "Funky" => "Andre",
              "SHiTmaN" => "Shitman",
	      "Legebarnet" => "[AD&D]Mc_Beat",
	      "Gunnar_Gedefjams" => "Funky_Boris",
	      "gf" => "Funky_Boris"
              );

############## end of configuration part ###############

sub new
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless ($self, $class);
    $self->_initialize(@_);
    return $self;
}

sub _initialize
{
    my $self = shift;
    $self->{in_dir} = shift;
    $self->reset();
}

sub timeToNumber
{
    my($self, $timestamp) = @_;

    my($date, $time);

    $_ = $timestamp;

    /(.*)\/(.*)\/(.*) - (.*):(.*):(.*)/ and do
    {
	$time = &Date_SecsSince1970($1, $2, $3, $4, $5, $6);
    };

    return $time;
}

sub buildStats
{
    my($self, $handle) = @_;

    my($killer, $victim, $weapon);
    my($user, $timestamp, $onlinetime,$luser);

    while($_ = <$handle>) {
	$luser = 0;
	if( /: \"([^<]*)([^ ]*)\" killed \"([^<]*)([^ ]*)\" with ([^ ]*)\n/
									   and do
									   {
									       $killer = $self->transPlayer($1);
									       $victim = $self->transPlayer($3);
									       $weapon = $5;
									   }
	  )
	{
	    ${$self->{score}}{$killer} += $points_per_kill;
	    ${$self->{weapon}}{$killer}->{$weapon}++;
	    ${$self->{killed}}{$killer}->{$victim}++;
	    ${$self->{killed_by}}{$victim}->{$killer}++;
	} if(/: \"([^<]*)([^ ]*)\" killed self with ([^ ]*)\n/ and do
							       {
								   $victim = $self->transPlayer($1);
								   $weapon = $3;
							       }
	    )
	{
	    ${$self->{score}}{$victim} += $points_for_suicide;
	    ${$self->{weapon}}{$victim}->{$weapon}++;
	} elsif( (/L (.*): \"([^<]*)([^ ]*)\" has entered the game/
								   and do {
								       $timestamp = $1;
								       $user = $self->transPlayer($2);
								   }
		 )
		 ||
		 (/L (.*): \"([^<]*)([^ ]*)\" changed name to \"([ ]*)([^<]*)([^ ]*)\"/
										       and do {
											   $timestamp = $1;
											   $luser = $self->transPlayer($2);
											   $user = $self->transPlayer($5);
										       }
		 )
	       )
	{
	    if($luser)
	    {
		$self-> disconnectUser($luser, $timestamp);
	    }

	    $self->connectUser($user, $timestamp);

	    if(!${$self->{score}}{$user})
	    {
		${$self->{score}}{$user} = 0;
	    }

	} elsif( /L (.*): \"([^<]*)([^ ]*)\" disconnected/
							  and do {
							      $timestamp = $1;
							      $user = $self->transPlayer($2);
							  }
	       )
	{

	    $self->disconnectUser($user, $timestamp);

	} elsif( (/L (.*): Spawning server/
					   and do {
					       $timestamp = $1;
					   }
		 ) ||
		 (/L (.*): Server shutdown/
					   and do {
					       $timestamp = $1;
					   }
		 )
	       )
	{
	    foreach my $tuser ( keys %{$self->{online}} )
	    {
		$self->disconnectUser($tuser, $timestamp);
	    }
	}
    }
}

sub connectUser
{
    my($self, $user, $timestamp) = @_;

    if( ${$self->{online}}{"$user"} )
    {
	print "$timestamp, $user allready connected!\n";
    }

    ${$self->{online}}{$user} = "$timestamp";
}

sub disconnectUser
{
    my($self, $user, $timestamp) = @_;

    my $onlinetime = 0;

    if(${$self->{online}}{$user})
    {
	#DEBUG
	#print "$user, from ${$self->{online}}{$user} to $timestamp\n";
	#print "$user, from " . $self->timeToNumber(${$self->{online}}{$user}) . " to " . $self->timeToNumber($timestamp) . "\n";
	$onlinetime = $self->timeToNumber($timestamp) - $self->timeToNumber(${$self->{online}}{$user});

	#DEBUG
	#print "$user before ${$self->{ontime}}{$user}\n";
	#END DEBUG

	${$self->{ontime}}{$user} += $onlinetime;

	#DEBUG
	#print "From ${$self->{online}}{$user} to $timestamp is $onlinetime\n";
	#print "$user +$onlinetime\n";
	#print "$user total time: ${$self->{ontime}}{$user}\n\n";
	#print "$user, -----------\n";
	#END DEBUG

	delete ${$self->{online}}{$user};
    } else
    {
	print "\nERROR $user not connected!!!\n";
    }
}

sub printKills
{
    my($self) = @_;

    print "Kills per user:\n";

    foreach my $user (keys %{$self->{score}})
    {
	print "\t$user\n";
	if($self->{killed_sorted})
	{
	    print map "$_: ${$self->{killed}}{$user}->{$_}\n", @{${$self->{killed_order}}{$user}};
	} else {
	    print map "$_: ${$self->{killed}}{$user}->{$_}\n", keys %{${$self->{killed}}{$user}};
	}
    }
}

sub sortKills
{
    my($self) = @_;
    foreach my $user (keys %{$self->{score}})
    {
	@{${$self->{killed_order}}{$user}} = sort {
	    ${$self->{killed}}{$user}->{$b} 
	    <=> 
	    ${$self->{killed}}{$user}->{$a}
	}
	keys %{ ${$self->{killed}}{$user} };
	
	@{${$self->{killed_by_order}}{$user}} = sort {
	    ${$self->{killed_by}}{$user}->{$b}
	    <=>
	    ${$self->{killed_by}}{$user}->{$a}
	}
	keys %{ ${$self->{killed_by}}{$user} };
    }
    $self->{killed_sorted} = 1;
    $self->{killed_by_sorted} = 1;
}

sub saveKills
{
    my($self, $dirname) = @_;

    my $FILE = new IO::File;
    my $FILE1 = new IO::File;

    mkdir $dirname, 0777;

    foreach my $user (keys %{$self->{score}})
    {
	open($FILE, ">$dirname/$user.kills") or die "Cannot open $dirname/$user.kills for writing";
	open($FILE1, ">$dirname/$user.killed_by") or die "Cannot open $dirname/$user.killed_by for writing";

	if($self->{killed_sorted})
	{
	    print $FILE map "$_: ${$self->{killed}}{$user}->{$_}\n", @{${$self->{killed_order}}{$user}};
	} else {
	    print $FILE map "$_: ${$self->{killed}}{$user}->{$_}\n", keys %{${$self->{killed}}{$user}};
	}

	if($self->{killed_by_sorted})
	{
	    print $FILE1 map "$_: ${$self->{killed_by}}{$user}->{$_}\n", @{${$self->{killed_by_order}}{$user}};
	} else {
	    print $FILE1 map "$_: ${$self->{killed_by}}{$user}->{$_}\n", keys %{${$self->{killed_by}}{$user}};
	}

	close($FILE);
	close($FILE1);
    }
}

sub reset
{
    my($self) = @_;
    %{$self->{score}} = ();
    %{$self->{weapon}} = ();
    @{$self->{order}} = ();

    $self->{sorted} = 0;

    %{$self->{killed}} = ();
    %{$self->{killed_by}} = ();
    %{$self->{killed_order}} = ();
    %{$self->{killed_by_order}} = ();
    $self->{killed_sorted} = 0;
    $self->{killed_by_sorted} = 0;

    $self->{ontime_sorted} = 0;

    @{$self->{ontime_order}} = ();

    %{$self->{online}} = ();
    %{$self->{ontime}} = ();
}

sub start
{
    my($self) = @_;

    my $dir = $self->{in_dir};

    opendir DIR, $dir;

    my @files = grep {/.log$/} readdir(DIR);
    @files = reverse @files;
    my $FILE = new IO::File;

    while($_ = pop(@files))
    {
	open($FILE, "$dir/$_") || die "Can't open log file";

	buildStats($self, $FILE);

	close $FILE;
    }
    
    closedir DIR;
}

sub printScores
{
    my($self) = @_;
    print "\n\nSCOREBOARD:\n";
    if($self->{sorted})
    {
	print map "$_: ${$self->{score}}{$_}\n", @{$self->{order}};
    } else {
	print map "$_: ${$self->{score}}{$_}\n", keys %{$self->{score}};
    }
}

sub saveScores
{
    my($self, $filename) = @_;

    my $SCORE = new IO::File;
    open($SCORE, ">$filename") or die "Cannot open $filename for writing";

    if($self->{sorted})
    {
	print $SCORE (map "$_: ${$self->{score}}{$_}\n", @{$self->{order}});
    } else {
	print $SCORE (map "$_: ${$self->{score}}{$_}\n", keys %{$self->{score}});
    }

    close $SCORE;
}

sub sortOnlineTimes
{
    my($self) = @_;

    @{ $self->{ontime_order} } =
    sort {
	${$self->{ontime}}{$b}
	<=>
        ${$self->{ontime}}{$a}
    }
    keys %{ $self->{ontime} };
    
    $self->{ontime_sorted} = 1;
}

sub printOnlineTimes
{
    my($self) = @_;
    if( $self->{ontime_sorted} )
    {
	print map "$_: ${$self->{ontime}}{$_}\n", @{ $self->{ontime_order} };
    } else {
	print map "$_: ${$self->{ontime}}{$_}\n", keys %{$self->{ontime}};
    }
}

sub saveOnlineTimes
{
    my($self, $filename) = @_;

    my $FILE = new IO::File;

    open($FILE, ">$filename") or die "Cannot open $filename for writing";

    if( $self->{ontime_sorted} )
    {
	print $FILE map "$_: ${$self->{ontime}}{$_}\n", @{ $self->{ontime_order} };
    } else {
	print $FILE map "$_: ${$self->{ontime}}{$_}\n", keys %{$self->{ontime}};
    }

    close $FILE;
}

sub transWeapon
{
  my($weapon) = @_;
  my $return = $weapon;

  if($trans{$weapon})
  {
    $return = $trans{$weapon};
  }

  return $return;
}
sub transPlayer
{
    my($self, $player) = @_;
    my $return = $player;

    if($alias{$player})
    {
	$return = $alias{$player};
    }

    return $return;
}

sub printWeaponScores
{
    my($self) = @_;
    print "\nWeapon List:\n";
    foreach my $user (keys %{$self->{weapon}})
    {
	print "$user:\n";
	if(${$self->{weapon}}{$user}->{order})
	{
	    print map "\t" . transWeapon($_) . ": ${$self->{weapon}}{$user}->{$_}\n", @{${$self->{weapon}}{$user}->{order}};
	} else {
	    print map "\t" . transWeapon($_) . ": ${$self->{weapon}}{$user}->{$_}\n", keys %{${$self->{weapon}}{$user}};
	}
    }
}

sub saveWeaponScores
{
    my($self, $filename) = @_;

    my $FILE = new IO::File;

    open $FILE, ">$filename" or die "Cannot open $filename for writing";

    foreach my $user (keys %{$self->{weapon}})
    {
	print $FILE "\t$user:\n";
	if(${$self->{weapon}}{$user}->{order})
	{
	    print $FILE map "" . transWeapon($_) . ":${$self->{weapon}}{$user}->{$_}\n", @{${$self->{weapon}}{$user}->{order}};
	} else {
	    print $FILE map "" . transWeapon($_) . ":${$self->{weapon}}{$user}->{$_}\n", keys %{${$self->{weapon}}{$user}};
	}
    }

    close $FILE;
}

sub sortScores
{
    my($self) = @_;
    @{$self->{order}} = sort { ${$self->{score}}{$b} <=> ${$self->{score}}{$a} } keys %{$self->{score}};
    $self->{sorted} = 1;
}

sub sortWeaponScores
{
    my($self) = @_;
    foreach my $user (keys %{$self->{weapon}})
    {
	my @worder = ();
	@worder = sort { ${$self->{weapon}}{$user}->{$b} <=> ${$self->{weapon}}{$user}->{$a} } keys %{${$self->{weapon}}{$user}};
	${$self->{weapon}} {$user}->{order} = \@worder;
    }
}

1;

__END__

=head1 NAME

ScoreBoard - A scoreboard system for halflife

=head1 SYNOPSIS

    use ScoreBoard;
    $b = new ScoreBoard $directory;

    $b->reset();
    $b->start();
    $b->sortScores();
    $b->saveScores($filename);

    $b->sortWeaponScores();
    $b->saveWeaponScores($filename);

    $b->sortOnlineTimes();
    $b->saveOnlineTimes($filename);

    $b->sortKills();
    $b->saveKills($dirname);

=head1 DESCRIPTION

ScoreBoard is a log analyzer, currently only for halflife.
Right now only data outputs are done, ie. they need to be prosessed by 
another program. Do please note that the different output methods have
different output formats (some output to files, others to directories).

=head2 METHODS

=over 4

=item B<new ScoreBoard $directory;>

Creates a new instance of ScoreBoard. B<$directory>
is where the log files are

=item B<reset();>

Resets all internal variables. It is automatically called by 
the new method

=item B<start();>

Analyzes all log files (*.log), and sets the internal variables.

=item B<sortScores();>

Sorts the array of scores, so that the player with highest score 
is listed first.

=item B<saveScores($filename);>

Saves the array of scores to $filename.
The format is:

"player: score\n"

where B<player> is the playername and B<score> is the score.

=item B<sortWeaponScores();>

Sorts the array of weapon scores for each user, so that the weapon
with most kills gets first.

=item B<saveWeaponScores($filename);>

Saves the weaponscores to $filename in the format:

\tplayer:\n
weapon: scores\n

=item B<sortOnlineTimes();>

Sorts the online times, so that the user that has played the most is 
the first.

=item B<saveOnlineTimes($filename);>

Saves the online times (or play times) array to $filename, which has 
the following format:

player: onlinetime\n

where the online time is in seconds.

=item B<sortKills();>

Sorts the "how many times did I kill who", and the "How many times was I killed by who" 
list for each player, so that the one who the player killed the most/the one who killed 
the player the most is on the top of the list.

=item B<saveKills($dirname);>

Saves the"how many times did I kill who" list, to $dirname/player.kills, in the format:

player_I_killed: number_of_times\n

And it saves the "How many times was I killed by" list to $dirname/player.killed_by in
the format:

player_who_killed_me: number_of_times\n


=back

=head1 LICENSE

This module is released under the Artistic License.

=head1 KNOWN BUGS

Weapon and user aliases are "hardcoded"

Points are "hardcoded"

=head1 TODO

Create some better output, HTML for instance.

Make the analyzing part more flexible so that it can handle log files without weapons logs.

Add support for other games

Aliases (weapon & user) have to be moved to a data file of some sort

Add support for ini files, or another way to remove the "hardcoded" stuff.

=head1 AUTHOR

Paul Fleischer, proguy@tpp.dhs.org

=cut
